
(* take a string and the list of all node and returns the position of the node in the list *)                          
let (pos:string -> Topology.node list -> int) =
  fun nid lid -> 
  fst (List.fold_left (fun (found_i,i) lid -> if (nid=lid.Topology.id) then (i,i+1) else (found_i,i+1) ) (-1, 0) lid)

(* take a graph t and returns the Adjacency matrix of t *)                                 
let (graph_to_adjency: Topology.t -> int array array) =
  fun t->
  let taille = List.length t.nodes in
  let mat = Array.make_matrix (taille) (taille) 0 in
  List.iter (fun n -> (List.iter (fun (_,m) ->  mat.(pos n.Topology.id t.nodes).(pos m t.nodes) <- 1 )
                         (t.succ n.Topology.id) ) ) (t.nodes);
  mat

    
(* Initialize the Adjacency matrix for Floyd Warshall algorithm *)
let (initFW: int array array -> int array array) =
  fun m ->
  let n = (Array.length m.(0)) in
  for i=0 to (n - 1) do
    for j=0 to (n - 1) do
      if (i<>j) then
        (if (m.(i).(j)=1) then m.(i).(j) <- 1 else m.(i).(j) <- n+1)
          else m.(i).(j) <- 0
      done;
  done;
  m


(* Apply Floyd Warshall algorithm which gives the matrix of all pairs shortest path *)
let (floydwarshall:int array array -> int array array) =
  fun m ->
  let w = initFW m in 
  let n = (Array.length m.(0)) in
  for k=0 to (n - 1) do
    for i=0 to (n - 1) do
      for j=0 to (n - 1) do
        w.(i).(j) <- (min (w.(i).(j)) (w.(i).(k) + w.(k).(j)))
        done;
      done;
  done;
  w


(* returns the greatest int of a matrix *)
let (max_mat: int array array -> int) =
  fun mat ->
  let n = (Array.length mat) and m = (Array.length mat.(0)) in
  let maxi = ref (-1) in
      for i=0 to (n - 1) do
        for j=0 to (m - 1) do
          maxi := max !maxi mat.(i).(j)
        done;
      done;
      !maxi

(* takes a graph t in argument and returns the diameter *)
let (get: Topology.t -> int ) =
  fun t ->
  let d = (max_mat(floydwarshall (graph_to_adjency t))) in
  Printf.eprintf " ====> The Graph Diameter is %d \n%!" d;
  d
