(*
Algorithm 1 of: 
  A Framework for Certified Self-Stabilization
  Case Study: Silent Self-Stabilizing k-Dominating Set on a Tree
https://hal.archives-ouvertes.fr/hal-01272158/document
*)

open Algo
open State
   

(* State predicates *)
  
let isRoot p = p.isRoot

let (isShort: 'st -> bool) =
  fun p -> p.alpha < k

let (isTall: 'st -> bool) =
  fun p -> p.alpha >= k

(* Actually unused *)
let (kDominator: 'v -> bool) = 
  fun p -> (p.alpha = k) || ((isShort p) && (isRoot p)) 


let rec (shortChildren: State.t neighbor list -> State.t list) =
  fun nl ->
  List.filter isShort (List.map state nl)

let rec (tallChildren: State.t neighbor list -> 'st list) =
  fun nl ->
  List.filter isTall (List.map state nl)

let rec (max: 'st list -> int -> int) =
  fun sl cur ->
  match sl with
    [] -> cur
  | s::liste -> if (s.alpha) > cur then max liste (s.alpha) else max liste cur
  
let rec (min: 'st list -> int -> int) =
  fun sl cur ->
  match sl with
    [] -> cur
  | s::liste -> if (s.alpha) < cur then min liste (s.alpha) else min liste cur
  
let (maxAShort: 'st neighbor list -> int) =
  fun nl -> max (shortChildren nl) (-1)

let (minATall: 'st neighbor list -> int) =
  fun nl -> min (tallChildren nl) (2*k+1)

let (newAlpha: 'st neighbor list -> int) =
  fun nl ->
  let mas = (maxAShort nl) in
  let mit = (minATall nl) in
  if (mas + mit) <= (2*k - 2) then (mit + 1) else (mas + 1)
  
(*end macros*)

let (init_state: int -> string -> 'st) =
  fun _ pid ->
  {
    isRoot = pid = "root"; (* ZZZ: The root of the tree should be named "root"! *)
    alpha = Random.int (2*k+1)
  }   
      
let (enable_f: 'st -> 'st neighbor list -> action list) =
  fun p nl -> if (p.alpha <> (newAlpha nl)) then ["change_alpha"] else []

let (step_f : 'st -> 'st neighbor list -> action -> 'st ) =
  fun p nl a -> if a = "change_alpha" then {p with alpha = (newAlpha nl)} else assert false
