open Algo
open P

let is_enabled_S p nl =
  P.enable_f p nl = ["S"]

let is_enabled_R p nl =
  let action = P.enable_f p nl in
  action = ["Rp"] || action = ["Rr"]

(** Potentiel d'un noeud, pour la règle R *)
let pot_r (p: State.t) (nl: State.t neighbor list) =
  if is_enabled_R p nl then Algo.sub_tree_size p.pid
  else 0

(** potentiel pour l'action "R" top-down:
   somme des tailles des sous-arbres des activables *)
let potential_r: State.t Algo.potential_fun =
  fun pidl get ->
    let potentials = List.map (fun pid ->
      match get pid with
      | (p, l) ->
        pot_r p (List.map fst l)
    ) pidl in
    float_of_int (List.fold_left (+) 0 potentials)

(** potentiel pour l'action S bottom-up
   somme des profondeurs des noeuds activables *)
let rec potential_s: State.t Algo.potential_fun =
  fun pidl get ->
    let levels = List.filter_map (fun pid ->
      match get pid with
      | (p, l) ->
        let nl = List.map fst l in
        if is_enabled_S p nl then
          Some (1 + Algo.level pid) (* 1 + niveau du noeud, pour que la racine ait une valeur de 1 *)
        else
          None
    ) pidl in
    float_of_int (List.fold_left (+) 0 levels)

let potential_combined: State.t Algo.potential_fun =
  fun pidl get ->
    let potR = potential_r pidl get in
    let potS = potential_s pidl get in
    (* Printf.printf "potR = %f; potS = %f\n" potR potS; *)
    let n = float_of_int (Algo.card ()) in
    let pot = (n**2.0 +. 1.0) *. potS +. potR in
    (* Printf.printf "potTotal = %f\n" pot; *)
    pot

let potential = Some potential_combined
let legitimate = None (* None => only silent configuration are legitimate *)
let fault = None (* None => the simulation stop once a legitimate configuration is reached *)
